@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2020 Lance Edgar
+#  Copyright © 2010-2021 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -31,7 +31,6 @@ import logging
 import six
 import humanize
 
-from rattail import pod
 from rattail.db import model
 from rattail.time import make_utc
 from rattail.util import pretty_quantity
@@ -126,13 +125,7 @@ class ReceivingBatchViews(APIBatchView):
         }
 
     def render_eligible_purchase(self, purchase):
-        if purchase.status == self.enum.PURCHASE_STATUS_ORDERED:
-            date = purchase.date_ordered
-            total = purchase.po_total
-        elif purchase.status == self.enum.PURCHASE_STATUS_RECEIVED:
-            date = purchase.date_received
-            total = purchase.invoice_total
-        return '{} for ${:0,.2f} ({})'.format(date, total or 0, purchase.department or purchase.buyer)
+        return self.handler.render_eligible_purchase(purchase)
 
     @classmethod
     def defaults(cls, config):
@@ -274,9 +267,12 @@ class ReceivingBatchRowViews(APIBatchRowView):
         return filters
 
     def normalize(self, row):
-        batch = row.batch
         data = super(ReceivingBatchRowViews, self).normalize(row)
 
+        batch = row.batch
+        app = self.get_rattail_app()
+        prodder = app.get_products_handler()
+
         data['product_uuid'] = row.product_uuid
         data['item_id'] = row.item_id
         data['upc'] = six.text_type(row.upc)
@@ -288,7 +284,7 @@ class ReceivingBatchRowViews(APIBatchRowView):
 
         # only provide image url if so configured
         if self.rattail_config.getbool('rattail.batch', 'purchase.mobile_images', default=True):
-            data['image_url'] = pod.get_image_url(self.rattail_config, row.upc) if row.upc else None
+            data['image_url'] = prodder.get_image_url(product=row.product, upc=row.upc)
 
         # unit_uom can vary by product
         data['unit_uom'] = 'LB' if row.product and row.product.weighed else 'EA'
