@@ -615,6 +615,9 @@ class Form(object):
         elif type_ == 'text':
             self.set_renderer(key, self.render_pre_sans_serif)
             self.set_widget(key, dfwidget.TextAreaWidget(cols=80, rows=8))
+        elif type_ == 'text_wrapped':
+            self.set_renderer(key, self.render_pre_sans_serif_wrapped)
+            self.set_widget(key, dfwidget.TextAreaWidget(cols=80, rows=8))
         elif type_ == 'file':
             tmpstore = SessionFileUploadTempStore(self.request)
             kw = {'widget': dfwidget.FileUploadWidget(tmpstore),
@@ -768,6 +771,7 @@ class Form(object):
         # TODO: deprecate / remove the latter option here
         if self.auto_disable_save or self.auto_disable:
             if self.use_buefy:
+                context['form_kwargs'].setdefault('ref', self.component_studly)
                 context['form_kwargs']['@submit'] = 'submit{}'.format(self.component_studly)
             else:
                 context['form_kwargs']['class_'] = 'autodisable'
@@ -914,14 +918,26 @@ class Form(object):
             return ""
         return HTML.tag('pre', value)
 
-    def render_pre_sans_serif(self, record, field_name):
+    def render_pre_sans_serif(self, record, field_name, wrapped=False):
         value = self.obtain_value(record, field_name)
         if value is None:
             return ""
-        # this uses a Bulma helper class, for which we also add custom styles
-        # to our "default" base.css (for jquery theme)
-        return HTML.tag('pre', class_='is-family-sans-serif',
-                        c=value)
+
+        kwargs = {
+            'c': value,
+            # this uses a Bulma helper class, for which we also add
+            # custom styles to our "default" base.css (for jquery
+            # theme)
+            'class_': 'is-family-sans-serif',
+        }
+
+        if wrapped:
+            kwargs['style'] = 'white-space: pre-wrap;'
+
+        return HTML.tag('pre', **kwargs)
+
+    def render_pre_sans_serif_wrapped(self, record, field_name):
+        return self.render_pre_sans_serif(record, field_name, wrapped=True)
 
     def obtain_value(self, record, field_name):
         if record:
