@@ -27,9 +27,11 @@ Core Grid Classes
 from __future__ import unicode_literals, absolute_import
 
 import datetime
-from six.moves import urllib
+import warnings
+import logging
 
 import six
+from six.moves import urllib
 import sqlalchemy as sa
 from sqlalchemy import orm
 
@@ -48,6 +50,9 @@ from tailbone.db import Session
 from tailbone.util import raw_datetime
 
 
+log = logging.getLogger(__name__)
+
+
 class FieldList(list):
     """
     Convenience wrapper for a field list.
@@ -69,12 +74,13 @@ class Grid(object):
 
     def __init__(self, key, data, columns=None, width='auto', request=None,
                  model_class=None, model_title=None, model_title_plural=None,
-                 enums={}, labels={}, assume_local_times=False, renderers={},
+                 enums={}, labels={}, assume_local_times=False, renderers={}, invisible=[],
                  extra_row_class=None, linked_columns=[], url='#',
                  joiners={}, filterable=False, filters={}, use_byte_string_filters=False,
                  sortable=False, sorters={}, default_sortkey=None, default_sortdir='asc',
                  pageable=False, default_pagesize=20, default_page=1,
                  checkboxes=False, checked=None, check_handler=None, check_all_handler=None,
+                 clicking_row_checks_box=False,
                  main_actions=[], more_actions=[], delete_speedbump=False,
                  ajax_data_url=None, component='tailbone-grid',
                  **kwargs):
@@ -104,6 +110,7 @@ class Grid(object):
         self.labels = labels or {}
         self.assume_local_times = assume_local_times
         self.renderers = self.make_default_renderers(renderers or {})
+        self.invisible = invisible or []
         self.extra_row_class = extra_row_class
         self.linked_columns = linked_columns or []
         self.url = url
@@ -128,6 +135,7 @@ class Grid(object):
             self.checked = lambda item: False
         self.check_handler = check_handler
         self.check_all_handler = check_all_handler
+        self.clicking_row_checks_box = clicking_row_checks_box
 
         self.main_actions = main_actions or []
         self.more_actions = more_actions or []
@@ -158,13 +166,45 @@ class Grid(object):
         mapper = orm.class_mapper(self.model_class)
         return [prop.key for prop in mapper.iterate_properties]
 
+    def remove(self, *keys):
+        """
+        This *removes* some column(s) from the grid, altogether.
+        """
+        for key in keys:
+            if key in self.columns:
+                self.columns.remove(key)
+
     def hide_column(self, key):
-        if key in self.columns:
-            self.columns.remove(key)
+        """
+        This *removes* a column from the grid, altogether.
+
+        This method is deprecated; use :meth:`remove()` instead.
+        """
+        warnings.warn("Grid.hide_column() is deprecated; please use "
+                      "Grid.remove() instead.",
+                      DeprecationWarning)
+        self.remove(key)
 
     def hide_columns(self, *keys):
-        for key in keys:
-            self.hide_column(key)
+        """
+        This *removes* columns from the grid, altogether.
+
+        This method is deprecated; use :meth:`remove()` instead.
+        """
+        self.remove(*keys)
+
+    def set_invisible(self, key, invisible=True):
+        """
+        Mark the given column as "invisible" (but do not remove it).
+
+        Use :meth:`hide_column()` if you actually want to remove it.
+        """
+        if invisible:
+            if key not in self.invisible:
+                self.invisible.append(key)
+        else:
+            if key in self.invisible:
+                self.invisible.remove(key)
 
     def append(self, field):
         self.columns.append(field)
@@ -1011,7 +1051,7 @@ class Grid(object):
 
         return render(template, context)
 
-    def set_filters_sequence(self, filters):
+    def set_filters_sequence(self, filters, only=False):
         """
         Explicitly set the sequence for grid filters, using the sequence
         provided.  If the grid currently has more filters than are mentioned in
@@ -1019,12 +1059,21 @@ class Grid(object):
         tacked on at the end.
 
         :param filters: Sequence of filter keys, i.e. field names.
+
+        :param only: If true, then *only* those filters specified will
+           be kept, and all others discarded.  If false then any
+           filters not specified will still be tacked onto the end, in
+           alphabetical order.
         """
         new_filters = gridfilters.GridFilterSet()
         for field in filters:
-            new_filters[field] = self.filters.pop(field)
-        for field in self.filters:
-            new_filters[field] = self.filters[field]
+            if field in self.filters:
+                new_filters[field] = self.filters.pop(field)
+            else:
+                log.warning("field '%s' is not in current filter set", field)
+        if not only:
+            for field in sorted(self.filters):
+                new_filters[field] = self.filters[field]
         self.filters = new_filters
 
     def get_filters_sequence(self):
@@ -1183,6 +1232,7 @@ class Grid(object):
                 'field': name,
                 'label': self.get_label(name),
                 'sortable': self.sortable and name in self.sorters,
+                'visible': name not in self.invisible,
             })
         return columns
 
@@ -1368,12 +1418,13 @@ class GridAction(object):
     """
 
     def __init__(self, key, label=None, url='#', icon=None, target=None,
-                 click_handler=None):
+                 link_class=None, click_handler=None):
         self.key = key
         self.label = label or prettify(key)
         self.icon = icon
         self.url = url
         self.target = target
+        self.link_class = link_class
         self.click_handler = click_handler
 
     def get_url(self, row, i):
@@ -1384,6 +1435,17 @@ class GridAction(object):
             return self.url(row, i)
         return self.url
 
+    def render_label(self):
+        """
+        Render the label "text" within the actions column of a grid
+        row.  Most actions have a static label that never varies, but
+        you can override this to add e.g. HTML content.  Note that the
+        return value will be treated / rendered as HTML whether or not
+        it contains any, so perhaps be careful that it is trusted
+        content.
+        """
+        return self.label
+
 
 class URLMaker(object):
     """
