@@ -32,7 +32,6 @@ import datetime
 
 import rattail
 from rattail.db import model
-from rattail.db.auth import cache_permissions
 
 import colander
 import deform
@@ -69,6 +68,7 @@ def new_request(event):
     """
     request = event.request
     rattail_config = request.registry.settings.get('rattail_config')
+    # TODO: why would this ever be null?
     if rattail_config:
         request.rattail_config = rattail_config
 
@@ -86,7 +86,18 @@ def new_request(event):
     request.is_admin = bool(request.user) and request.user.is_admin()
     request.is_root = request.is_admin and request.session.get('is_root', False)
 
-    request.tailbone_cached_permissions = cache_permissions(Session(), request.user)
+    if rattail_config:
+        app = rattail_config.get_app()
+        auth = app.get_auth_handler()
+        request.tailbone_cached_permissions = auth.cache_permissions(
+            Session(), request.user)
+    # TODO: until we know otherwise, let's assume this is not needed
+    # else:
+    #     # TODO: not sure why this would really work, or even be
+    #     # needed, if there was no rattail config?
+    #     from rattail.db.auth import cache_permissions
+    #     request.tailbone_cached_permissions = cache_permissions(
+    #         Session(), request.user)
 
 
 def before_render(event):
