@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2020 Lance Edgar
+#  Copyright © 2010-2021 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -55,11 +55,11 @@ class CommonView(View):
     project_version = tailbone.__version__
     robots_txt_path = resource_path('tailbone.static:robots.txt')
 
-    def home(self, mobile=False):
+    def home(self, **kwargs):
         """
         Home page view.
         """
-        if not mobile and not self.request.user:
+        if not self.request.user:
             if self.rattail_config.getbool('tailbone', 'login_is_home', default=True):
                 raise self.redirect(self.request.route_url('login'))
 
@@ -96,12 +96,6 @@ class CommonView(View):
             response.content_type = b'text/plain'
         return response
 
-    def mobile_home(self):
-        """
-        Home page view for mobile.
-        """
-        return self.home(mobile=True)
-
     def exception(self):
         """
         Generic exception view
@@ -112,12 +106,16 @@ class CommonView(View):
         """
         Generic view to show "about project" info page.
         """
-        return {
+        use_buefy = self.get_use_buefy()
+        context = {
             'project_title': self.project_title,
             'project_version': self.project_version,
             'packages': self.get_packages(),
-            'use_buefy': self.get_use_buefy(),
+            'use_buefy': use_buefy,
         }
+        if use_buefy:
+            context['index_title'] = self.rattail_config.node_title()
+        return context
 
     def get_packages(self):
         """
@@ -175,12 +173,6 @@ class CommonView(View):
             return {'ok': True}
         return {'error': "Form did not validate!"}
 
-    def mobile_feedback(self):
-        """
-        Generic view to handle the user feedback form on mobile.
-        """
-        return self.feedback()
-
     def consume_batch_id(self):
         """
         Consume next batch ID from the PG sequence, and display via flash message.
@@ -203,7 +195,6 @@ class CommonView(View):
     @classmethod
     def _defaults(cls, config):
         rattail_config = config.registry.settings.get('rattail_config')
-        legacy_mobile = cls.legacy_mobile_enabled(rattail_config)
 
         # auto-correct URLs which require trailing slash
         config.add_notfound_view(cls, attr='notfound', append_slash=True)
@@ -218,9 +209,6 @@ class CommonView(View):
         # home
         config.add_route('home', '/')
         config.add_view(cls, attr='home', route_name='home', renderer='/home.mako')
-        if legacy_mobile:
-            config.add_route('mobile.home', '/mobile/')
-            config.add_view(cls, attr='mobile_home', route_name='mobile.home', renderer='/mobile/home.mako')
 
         # robots.txt
         config.add_route('robots.txt', '/robots.txt')
@@ -229,9 +217,6 @@ class CommonView(View):
         # about
         config.add_route('about', '/about')
         config.add_view(cls, attr='about', route_name='about', renderer='/about.mako')
-        if legacy_mobile:
-            config.add_route('mobile.about', '/mobile/about')
-            config.add_view(cls, attr='about', route_name='mobile.about', renderer='/mobile/about.mako')
 
         # change db engine
         config.add_tailbone_permission('common', 'common.change_db_engine',
@@ -251,10 +236,6 @@ class CommonView(View):
         config.add_route('feedback', '/feedback', request_method='POST')
         config.add_view(cls, attr='feedback', route_name='feedback',
                         renderer='json', permission='common.feedback')
-        if legacy_mobile:
-            config.add_route('mobile.feedback', '/mobile/feedback', request_method='POST')
-            config.add_view(cls, attr='mobile_feedback', route_name='mobile.feedback',
-                            renderer='json', permission='common.feedback')
 
         # consume batch ID
         config.add_tailbone_permission('common', 'common.consume_batch_id',
