@@ -26,11 +26,8 @@ Customer Views
 
 from __future__ import unicode_literals, absolute_import
 
-import re
-
 import six
 import sqlalchemy as sa
-from sqlalchemy import orm
 
 import colander
 from pyramid.httpexceptions import HTTPNotFound
@@ -38,7 +35,7 @@ from webhelpers2.html import HTML, tags
 
 from tailbone import grids
 from tailbone.db import Session
-from tailbone.views import MasterView, AutocompleteView
+from tailbone.views import MasterView
 
 from rattail.db import model
 
@@ -52,6 +49,7 @@ class CustomerView(MasterView):
     has_versions = True
     people_detachable = True
     touchable = True
+    supports_autocomplete = True
 
     # whether to show "view full profile" helper for customer view
     show_profiles_helper = True
@@ -173,6 +171,7 @@ class CustomerView(MasterView):
         super(CustomerView, self).configure_common_form(f)
         customer = f.model_instance
         permission_prefix = self.get_permission_prefix()
+        use_buefy = self.get_use_buefy()
 
         f.set_renderer('default_email', self.render_default_email)
         if not self.creating and customer.emails:
@@ -217,13 +216,15 @@ class CustomerView(MasterView):
             f.set_renderer('person', self.form_render_person)
 
         # people
-        if self.creating:
-            f.remove_field('people')
-        elif self.viewing and self.request.has_perm('{}.detach_person'.format(permission_prefix)):
-            f.set_renderer('people', self.render_people_removable)
+        if self.viewing:
+            if use_buefy:
+                f.set_renderer('people', self.render_people_buefy)
+            elif self.people_detachable and self.has_perm('detach_person'):
+                f.set_renderer('people', self.render_people_removable)
+            else:
+                f.set_renderer('people', self.render_people)
         else:
-            f.set_renderer('people', self.render_people)
-            f.set_readonly('people')
+            f.remove('people')
 
         # groups
         if self.creating:
@@ -245,7 +246,35 @@ class CustomerView(MasterView):
             f.set_readonly('members')
 
     def template_kwargs_view(self, **kwargs):
+        kwargs = super(CustomerView, self).template_kwargs_view(**kwargs)
+
         kwargs['show_profiles_helper'] = self.show_profiles_helper
+
+        use_buefy = self.get_use_buefy()
+        if use_buefy:
+            customer = kwargs['instance']
+            people = []
+            for person in customer.people:
+                data = {
+                    'uuid': person.uuid,
+                    'full_name': person.display_name,
+                    'first_name': person.first_name,
+                    'last_name': person.last_name,
+                    '_action_url_view': self.request.route_url('people.view',
+                                                               uuid=person.uuid),
+                }
+                if self.editable and self.request.has_perm('people.edit'):
+                    data['_action_url_edit'] = self.request.route_url(
+                        'people.edit',
+                        uuid=person.uuid)
+                if self.people_detachable and self.has_perm('detach_person'):
+                    data['_action_url_detach'] = self.request.route_url(
+                        'customers.detach_person',
+                        uuid=customer.uuid,
+                        person_uuid=person.uuid)
+                people.append(data)
+            kwargs['people_data'] = people
+
         return kwargs
 
     def unique_id(self, node, value):
@@ -318,6 +347,35 @@ class CustomerView(MasterView):
             main_actions=actions)
         return HTML.literal(g.render_grid())
 
+    def render_people_buefy(self, customer, field):
+        route_prefix = self.get_route_prefix()
+        permission_prefix = self.get_permission_prefix()
+
+        factory = self.get_grid_factory()
+        g = factory(
+            key='{}.people'.format(route_prefix),
+            data=[],
+            columns=[
+                'full_name',
+                'first_name',
+                'last_name',
+            ],
+            sortable=True,
+            sorters={'full_name': True, 'first_name': True, 'last_name': True},
+        )
+
+        if self.request.has_perm('people.view'):
+            g.main_actions.append(self.make_action('view', icon='eye'))
+        if self.request.has_perm('people.edit'):
+            g.main_actions.append(self.make_action('edit', icon='edit'))
+        if self.people_detachable and self.has_perm('detach_person'):
+            g.main_actions.append(self.make_action('detach', icon='minus-circle',
+                                                   link_class='has-text-warning',
+                                                   click_handler="$emit('detach-person', props.row._action_url_detach)"))
+
+        return HTML.literal(
+            g.render_buefy_table_element(data_prop='peopleData'))
+
     def render_groups(self, customer, field):
         groups = customer.groups
         if not groups:
@@ -372,22 +430,87 @@ class CustomerView(MasterView):
     def _customer_defaults(cls, config):
         route_prefix = cls.get_route_prefix()
         url_prefix = cls.get_url_prefix()
+        instance_url_prefix = cls.get_instance_url_prefix()
         permission_prefix = cls.get_permission_prefix()
         model_key = cls.get_model_key()
         model_title = cls.get_model_title()
 
         # detach person
         if cls.people_detachable:
-            config.add_tailbone_permission(permission_prefix, '{}.detach_person'.format(permission_prefix),
+            config.add_tailbone_permission(permission_prefix,
+                                           '{}.detach_person'.format(permission_prefix),
                                            "Detach a Person from a {}".format(model_title))
-            config.add_route('{}.detach_person'.format(route_prefix), '{}/{{{}}}/detach-person/{{person_uuid}}'.format(url_prefix, model_key),
+            # TODO: this should require POST, but we'll add that once
+            # we can assume a Buefy theme is present, to avoid having
+            # to implement the logic in old jquery...
+            config.add_route('{}.detach_person'.format(route_prefix),
+                             '{}/detach-person/{{person_uuid}}'.format(instance_url_prefix),
                              # request_method='POST',
             )
-            config.add_view(cls, attr='detach_person', route_name='{}.detach_person'.format(route_prefix),
+            config.add_view(cls, attr='detach_person',
+                            route_name='{}.detach_person'.format(route_prefix),
                             permission='{}.detach_person'.format(permission_prefix))
 
-# TODO: deprecate / remove this
-CustomersView = CustomerView
+
+class PendingCustomerView(MasterView):
+    """
+    Master view for the Pending Customer class.
+    """
+    model_class = model.PendingCustomer
+    route_prefix = 'pending_customers'
+    url_prefix = '/customers/pending'
+
+    labels = {
+        'id': "ID",
+        'status_code': "Status",
+    }
+
+    grid_columns = [
+        'id',
+        'display_name',
+        'first_name',
+        'last_name',
+        'phone_number',
+        'email_address',
+        'status_code',
+    ]
+
+    form_fields = [
+        'id',
+        'display_name',
+        'first_name',
+        'middle_name',
+        'last_name',
+        'phone_number',
+        'phone_type',
+        'email_address',
+        'email_type',
+        'address_street',
+        'address_street2',
+        'address_city',
+        'address_state',
+        'address_zipcode',
+        'address_type',
+        'status_code',
+        'created',
+        'user',
+    ]
+
+    def configure_grid(self, g):
+        super(PendingCustomerView, self).configure_grid(g)
+
+        g.set_enum('status_code', self.enum.PENDING_CUSTOMER_STATUS)
+
+        g.set_sort_defaults('display_name')
+        g.set_link('id')
+        g.set_link('display_name')
+
+    def configure_form(self, f):
+        super(PendingCustomerView, self).configure_form(f)
+
+        f.set_enum('status_code', self.enum.PENDING_CUSTOMER_STATUS)
+
+        f.set_renderer('user', self.render_user)
 
 
 # # TODO: this is referenced by some custom apps, but should be moved??
@@ -408,43 +531,6 @@ def unique_id(node, value):
         raise colander.Invalid(node, "Customer ID must be unique")
 
 
-class CustomerNameAutocomplete(AutocompleteView):
-    """
-    Autocomplete view which operates on customer name.
-    """
-    mapped_class = model.Customer
-    fieldname = 'name'
-
-
-class CustomerPhoneAutocomplete(AutocompleteView):
-    """
-    Autocomplete view which operates on customer phone number.
-
-    .. note::
-       As currently implemented, this view will only work with a PostgreSQL
-       database.  It normalizes the user's search term and the database values
-       to numeric digits only (i.e. removes special characters from each) in
-       order to be able to perform smarter matching.  However normalizing the
-       database value currently uses the PG SQL ``regexp_replace()`` function.
-    """
-    invalid_pattern = re.compile(r'\D')
-
-    def prepare_term(self, term):
-        return self.invalid_pattern.sub('', term)
-
-    def query(self, term):
-        return Session.query(model.CustomerPhoneNumber)\
-            .filter(sa.func.regexp_replace(model.CustomerPhoneNumber.number, r'\D', '', 'g').like('%{0}%'.format(term)))\
-            .order_by(model.CustomerPhoneNumber.number)\
-            .options(orm.joinedload(model.CustomerPhoneNumber.customer))
-
-    def display(self, phone):
-        return "{0} {1}".format(phone.number, phone.customer)
-
-    def value(self, phone):
-        return phone.customer.uuid
-
-
 def customer_info(request):
     """
     View which returns simple dictionary of info for a particular customer.
@@ -462,17 +548,10 @@ def customer_info(request):
 
 def includeme(config):
 
-    # autocomplete
-    config.add_route('customers.autocomplete', '/customers/autocomplete')
-    config.add_view(CustomerNameAutocomplete, route_name='customers.autocomplete',
-                    renderer='json', permission='customers.list')
-    config.add_route('customers.autocomplete.phone', '/customers/autocomplete/phone')
-    config.add_view(CustomerPhoneAutocomplete, route_name='customers.autocomplete.phone',
-                    renderer='json', permission='customers.list')
-
     # info
     config.add_route('customer.info', '/customers/info')
     config.add_view(customer_info, route_name='customer.info',
                     renderer='json', permission='customers.view')
 
     CustomerView.defaults(config)
+    PendingCustomerView.defaults(config)
