@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2020 Lance Edgar
+#  Copyright © 2010-2021 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -31,6 +31,7 @@ import six
 from rattail.db import model
 
 import colander
+from webhelpers2.html import tags
 
 from tailbone import forms
 from tailbone.views.batch import BatchMasterView
@@ -47,19 +48,25 @@ class CustomerOrderBatchView(BatchMasterView):
 
     grid_columns = [
         'id',
-        'customer',
+        'contact_name',
         'rowcount',
         'total_price',
         'created',
         'created_by',
+        'executed',
+        'executed_by',
     ]
 
     form_fields = [
         'id',
+        'store',
         'customer',
         'person',
+        'pending_customer',
+        'contact_name',
         'phone_number',
         'email_address',
+        'params',
         'created',
         'created_by',
         'rowcount',
@@ -82,6 +89,7 @@ class CustomerOrderBatchView(BatchMasterView):
         'product_size',
         'order_quantity',
         'order_uom',
+        'case_quantity',
         'total_price',
         'status_code',
     ]
@@ -91,7 +99,7 @@ class CustomerOrderBatchView(BatchMasterView):
 
         g.set_type('total_price', 'currency')
 
-        g.set_link('customer')
+        g.set_link('contact_name')
         g.set_link('created')
         g.set_link('created_by')
 
@@ -104,6 +112,8 @@ class CustomerOrderBatchView(BatchMasterView):
         f.set_readonly('rows')
         f.set_readonly('status_code')
 
+        f.set_renderer('store', self.render_store)
+
         # customer
         if 'customer' in f.fields and self.editing:
             f.replace('customer', 'customer_uuid')
@@ -144,8 +154,19 @@ class CustomerOrderBatchView(BatchMasterView):
         else:
             f.set_renderer('person', self.render_person)
 
+        # pending_customer
+        f.set_renderer('pending_customer', self.render_pending_customer)
+
         f.set_type('total_price', 'currency')
 
+    def render_pending_customer(self, batch, field):
+        pending = batch.pending_customer
+        if not pending:
+            return
+        text = six.text_type(pending)
+        url = self.request.route_url('pending_customers.view', uuid=pending.uuid)
+        return tags.link_to(text, url)
+
     def row_grid_extra_class(self, row, i):
         if row.status_code == row.STATUS_PRODUCT_NOT_FOUND:
             return 'warning'
