@@ -51,7 +51,7 @@ from webhelpers2.html import tags, HTML
 
 from tailbone import forms, grids
 from tailbone.db import Session
-from tailbone.views import MasterView, AutocompleteView
+from tailbone.views import MasterView
 from tailbone.util import raw_datetime
 
 
@@ -84,6 +84,7 @@ class ProductView(MasterView):
     model_class = model.Product
     has_versions = True
     results_downloadable_xlsx = True
+    supports_autocomplete = True
 
     labels = {
         'item_id': "Item ID",
@@ -92,6 +93,8 @@ class ProductView(MasterView):
         'tax1': "Tax 1",
         'tax2': "Tax 2",
         'tax3': "Tax 3",
+        'tpr_price': "TPR Price",
+        'tpr_price_ends': "TPR Price Ends",
     }
 
     grid_columns = [
@@ -130,6 +133,10 @@ class ProductView(MasterView):
         'regular_price',
         'current_price',
         'current_price_ends',
+        'sale_price',
+        'sale_price_ends',
+        'tpr_price',
+        'tpr_price_ends',
         'vendor',
         'cost',
         'deposit_link',
@@ -166,11 +173,16 @@ class ProductView(MasterView):
     # same, but for prices
     RegularPrice = orm.aliased(model.ProductPrice)
     CurrentPrice = orm.aliased(model.ProductPrice)
+    SalePrice = orm.aliased(model.ProductPrice)
+    TPRPrice = orm.aliased(model.ProductPrice)
 
     def __init__(self, request):
         super(ProductView, self).__init__(request)
         self.print_labels = request.rattail_config.getbool('tailbone', 'products.print_labels', default=False)
 
+        app = self.get_rattail_app()
+        self.handler = app.get_products_handler()
+
     def query(self, session):
         user = self.request.user
         if user and user not in session:
@@ -199,6 +211,8 @@ class ProductView(MasterView):
 
     def configure_grid(self, g):
         super(ProductView, self).configure_grid(g)
+        app = self.get_rattail_app()
+        use_buefy = self.get_use_buefy()
 
         def join_vendor(q):
             return q.outerjoin(self.ProductVendorCost,
@@ -227,8 +241,24 @@ class ProductView(MasterView):
                                ProductCostCodeAny.product_uuid == model.Product.uuid)
 
         g.joiners['brand'] = lambda q: q.outerjoin(model.Brand)
-        g.joiners['department'] = lambda q: q.outerjoin(model.Department,
-                                                        model.Department.uuid == model.Product.department_uuid)
+
+        # department
+        g.set_joiner('department', lambda q: q.outerjoin(model.Department))
+        g.set_sorter('department', model.Department.name)
+        department_choices = app.cache_model(self.Session(), model.Department,
+                                             order_by=model.Department.name,
+                                             normalizer=lambda d: d.name)
+        department_choices = OrderedDict([('', "(any)")]
+                                         + sorted(six.iteritems(department_choices),
+                                                  key=lambda itm: itm[1]))
+        if not use_buefy:
+            department_choices = [tags.Option(name, uuid)
+                                  for uuid, name in six.iteritems(department_choices)]
+        g.set_filter('department', model.Department.uuid,
+                     value_enum=department_choices,
+                     verbs=['equal', 'not_equal', 'is_null', 'is_not_null', 'is_any'],
+                     default_active=True, default_verb='equal')
+
         g.joiners['subdepartment'] = lambda q: q.outerjoin(model.Subdepartment,
                                                            model.Subdepartment.uuid == model.Product.subdepartment_uuid)
         g.joiners['code'] = lambda q: q.outerjoin(model.ProductCode)
@@ -238,7 +268,6 @@ class ProductView(MasterView):
         g.joiners['vendor_code_any'] = join_vendor_code_any
 
         g.sorters['brand'] = g.make_sorter(model.Brand.name)
-        g.sorters['department'] = g.make_sorter(model.Department.name)
         g.sorters['subdepartment'] = g.make_sorter(model.Subdepartment.name)
         g.sorters['vendor'] = g.make_sorter(model.Vendor.name)
 
@@ -271,8 +300,6 @@ class ProductView(MasterView):
         g.filters['description'].default_verb = 'contains'
         g.filters['brand'] = g.make_filter('brand', model.Brand.name,
                                            default_active=True, default_verb='contains')
-        g.filters['department'] = g.make_filter('department', model.Department.name,
-                                                default_active=True, default_verb='contains')
         g.filters['subdepartment'] = g.make_filter('subdepartment', model.Subdepartment.name)
         g.filters['code'] = g.make_filter('code', model.ProductCode.code)
         g.filters['vendor'] = g.make_filter('vendor', model.Vendor.name)
@@ -302,6 +329,16 @@ class ProductView(MasterView):
         g.set_sorter('current_price', self.CurrentPrice.price)
         g.set_filter('current_price', self.CurrentPrice.price, label="Current Price")
 
+        # tpr_price
+        g.set_joiner('tpr_price', lambda q: q.outerjoin(
+            self.TPRPrice, self.TPRPrice.uuid == model.Product.tpr_price_uuid))
+        g.set_filter('tpr_price', self.TPRPrice.price)
+
+        # sale_price
+        g.set_joiner('sale_price', lambda q: q.outerjoin(
+            self.SalePrice, self.SalePrice.uuid == model.Product.sale_price_uuid))
+        g.set_filter('sale_price', self.SalePrice.price)
+
         # suggested_price
         g.set_renderer('suggested_price', self.render_grid_suggested_price)
 
@@ -408,6 +445,34 @@ class ProductView(MasterView):
             f.set_readonly('current_price_ends')
             f.set_renderer('current_price_ends', self.render_current_price_ends)
 
+        # sale_price
+        if self.creating:
+            f.remove_field('sale_price')
+        else:
+            f.set_readonly('sale_price')
+            f.set_renderer('sale_price', self.render_price)
+
+        # sale_price_ends
+        if self.creating:
+            f.remove_field('sale_price_ends')
+        else:
+            f.set_readonly('sale_price_ends')
+            f.set_renderer('sale_price_ends', self.render_sale_price_ends)
+
+        # tpr_price
+        if self.creating:
+            f.remove_field('tpr_price')
+        else:
+            f.set_readonly('tpr_price')
+            f.set_renderer('tpr_price', self.render_price)
+
+        # tpr_price_ends
+        if self.creating:
+            f.remove_field('tpr_price_ends')
+        else:
+            f.set_readonly('tpr_price_ends')
+            f.set_renderer('tpr_price_ends', self.render_tpr_price_ends)
+
         # vendor
         if self.creating:
             f.remove_field('vendor')
@@ -453,27 +518,14 @@ class ProductView(MasterView):
             return ""
         return "${:0.2f}".format(cost.unit_cost)
 
-    def render_price(self, product, column):
-        price = product[column]
-        if price:
-            if not product.not_for_sale:
-                if price.price is not None and price.pack_price is not None:
-                    if price.multiple > 1:
-                        return HTML.literal("$ {:0.2f} / {}&nbsp; ($ {:0.2f} / {})".format(
-                            price.price, price.multiple,
-                            price.pack_price, price.pack_multiple))
-                    return HTML.literal("$ {:0.2f}&nbsp; ($ {:0.2f} / {})".format(
-                        price.price, price.pack_price, price.pack_multiple))
-                if price.price is not None:
-                    if price.multiple is not None and price.multiple > 1:
-                        return "$ {:0.2f} / {}".format(price.price, price.multiple)
-                    return "$ {:0.2f}".format(price.price)
-                if price.pack_price is not None:
-                    return "$ {:0.2f} / {}".format(price.pack_price, price.pack_multiple)
-        return ""
+    def render_price(self, product, field):
+        if not product.not_for_sale:
+            price = product[field]
+            if price:
+                return self.handler.render_price(price)
         
     def render_current_price_for_grid(self, product, field):
-        text = self.render_price(product, field)
+        text = self.render_price(product, field) or ""
 
         price = product.current_price
         if price:
@@ -552,8 +604,10 @@ class ProductView(MasterView):
 
     def render_suggested_price(self, product, column):
         text = self.render_price(product, column)
+        if not text:
+            return
 
-        if text and self.show_price_effective_dates():
+        if self.show_price_effective_dates():
             history = self.get_suggested_price_history(product)
             if history:
                 date = localtime(self.rattail_config, history[0]['changed'], from_utc=True).date()
@@ -657,6 +711,7 @@ class ProductView(MasterView):
         fields.append('vendor_name')
         fields.append('vendor_item_code')
         fields.append('unit_cost')
+        fields.append('true_margin')
 
         return fields
 
@@ -718,8 +773,23 @@ class ProductView(MasterView):
         if 'unit_cost' in fields:
             row['unit_cost'] = product.cost.unit_cost if product.cost else None
 
+        if 'true_margin' in fields:
+            row['true_margin'] = None
+            if product.volatile and product.volatile.true_margin:
+                row['true_margin'] = product.volatile.true_margin
+
         return row
 
+    def download_results_normalize(self, product, fields, **kwargs):
+        data = super(ProductView, self).download_results_normalize(
+            product, fields, **kwargs)
+
+        if 'upc' in data:
+            if isinstance(data['upc'], GPC):
+                data['upc'] = six.text_type(data['upc'])
+
+        return data
+
     def get_instance(self):
         key = self.request.matchdict['uuid']
         product = Session.query(model.Product).get(key)
@@ -987,6 +1057,22 @@ class ProductView(MasterView):
             return ""
         return raw_datetime(self.request.rattail_config, value)
 
+    def render_sale_price_ends(self, product, field):
+        if not product.sale_price:
+            return
+        ends = product.sale_price.ends
+        if not ends:
+            return
+        return raw_datetime(self.rattail_config, ends)
+
+    def render_tpr_price_ends(self, product, field):
+        if not product.tpr_price:
+            return
+        ends = product.tpr_price.ends
+        if not ends:
+            return
+        return raw_datetime(self.rattail_config, ends)
+
     def render_inventory_on_hand(self, product, field):
         if not product.inventory:
             return ""
@@ -1061,23 +1147,7 @@ class ProductView(MasterView):
         product = kwargs['instance']
         use_buefy = self.get_use_buefy()
 
-        # TODO: pretty sure this is no longer needed?  guess we'll find out
-        # kwargs['image'] = False
-
-        # maybe provide image URL for product; we prefer image from our DB if
-        # present, but otherwise a "POD" image URL can be attempted.
-        if product.image:
-            kwargs['image_url'] = self.request.route_url('products.image', uuid=product.uuid)
-
-        elif product.upc:
-            if self.rattail_config.getbool('tailbone', 'products.show_pod_image', default=False):
-                # here we try to give a URL to a so-called "POD" image for the product
-                kwargs['image_url'] = pod.get_image_url(self.rattail_config, product.upc)
-                kwargs['image_path'] = pod.get_image_path(self.rattail_config, product.upc)
-
-        # maybe use "image not found" placeholder image
-        if not kwargs.get('image_url'):
-            kwargs['image_url'] = self.request.static_url('tailbone:static/img/product.png')
+        kwargs['image_url'] = self.handler.get_image_url(product)
 
         # add price history, if user has access
         if self.rattail_config.versioning_enabled() and self.has_perm('versions'):
@@ -1162,8 +1232,88 @@ class ProductView(MasterView):
         kwargs['costs_label_vendor'] = "Vendor"
         kwargs['costs_label_code'] = "Order Code"
         kwargs['costs_label_case_size'] = "Case Size"
+
+        if use_buefy:
+            kwargs['vendor_sources'] = self.get_context_vendor_sources(product)
+            kwargs['lookup_codes'] = self.get_context_lookup_codes(product)
+
         return kwargs
 
+    def get_context_vendor_sources(self, product):
+        app = self.get_rattail_app()
+        route_prefix = self.get_route_prefix()
+
+        factory = self.get_grid_factory()
+        g = factory(
+            key='{}.vendor_sources'.format(route_prefix),
+            data=[],
+            columns=[
+                'preferred',
+                'vendor',
+                'vendor_item_code',
+                'case_size',
+                'case_cost',
+                'unit_cost',
+                'status',
+            ],
+            labels={
+                'preferred': "Pref.",
+                'vendor_item_code': "Order Code",
+            },
+        )
+
+        sources = []
+        link_vendor = self.request.has_perm('vendors.view')
+        for cost in product.costs:
+
+            source = {
+                'uuid': cost.uuid,
+                'preferred': "X" if cost.preference == 1 else None,
+                'vendor_item_code': cost.code,
+                'case_size': app.render_quantity(cost.case_size),
+                'case_cost': app.render_currency(cost.case_cost),
+                'unit_cost': app.render_currency(cost.unit_cost, scale=4),
+                'status': "discontinued" if cost.discontinued else "available",
+            }
+
+            text = six.text_type(cost.vendor)
+            if link_vendor:
+                url = self.request.route_url('vendors.view', uuid=cost.vendor.uuid)
+                source['vendor'] = tags.link_to(text, url)
+            else:
+                source['vendor'] = text
+
+            sources.append(source)
+
+        return {'grid': g, 'data': sources}
+
+    def get_context_lookup_codes(self, product):
+        route_prefix = self.get_route_prefix()
+
+        factory = self.get_grid_factory()
+        g = factory(
+            key='{}.lookup_codes'.format(route_prefix),
+            data=[],
+            columns=[
+                'sequence',
+                'code',
+            ],
+            labels={
+                'sequence': "Seq.",
+            },
+        )
+
+        lookup_codes = []
+        for code in product._codes:
+
+            lookup_codes.append({
+                'uuid': code.uuid,
+                'sequence': code.ordinal,
+                'code': code.code,
+            })
+
+        return {'grid': g, 'data': lookup_codes}
+
     def get_regular_price_history(self, product):
         """
         Returns a sequence of "records" which corresponds to the given
@@ -1808,30 +1958,164 @@ class ProductView(MasterView):
                         renderer='json',
                         permission='{}.versions'.format(permission_prefix))
 
-# TODO: deprecate / remove this
-ProductsView = ProductView
 
-
-class ProductsAutocomplete(AutocompleteView):
+class PendingProductView(MasterView):
     """
-    Autocomplete view for products.
+    Master view for the Pending Product class.
     """
-    mapped_class = model.Product
-    fieldname = 'description'
+    model_class = model.PendingProduct
+    route_prefix = 'pending_products'
+    url_prefix = '/products/pending'
 
-    def query(self, term):
-        q = Session.query(model.Product).outerjoin(model.Brand)
-        q = q.filter(sa.or_(
-                model.Brand.name.ilike('%{}%'.format(term)),
-                model.Product.description.ilike('%{}%'.format(term))))
-        if not self.request.has_perm('products.view_deleted'):
-            q = q.filter(model.Product.deleted == False)
-        q = q.order_by(model.Brand.name, model.Product.description)
-        q = q.options(orm.joinedload(model.Product.brand))
-        return q
+    labels = {
+        'regular_price_amount': "Regular Price",
+        'status_code': "Status",
+        'user': "Created by",
+    }
 
-    def display(self, product):
-        return product.full_description
+    grid_columns = [
+        '_product_key_',
+        'department_name',
+        'brand_name',
+        'description',
+        'size',
+        'created',
+        'user',
+        'status_code',
+    ]
+
+    form_fields = [
+        '_product_key_',
+        'department_name',
+        'department',
+        'brand_name',
+        'brand',
+        'description',
+        'size',
+        'case_size',
+        'regular_price_amount',
+        'special_order',
+        'notes',
+        'created',
+        'user',
+        'status_code',
+    ]
+
+    def configure_grid(self, g):
+        super(PendingProductView, self).configure_grid(g)
+
+        # product key
+        if '_product_key_' in g.columns:
+            key = self.rattail_config.product_key()
+            g.replace('_product_key_', key)
+            g.set_label(key, self.rattail_config.product_key_title(key))
+            g.set_link(key)
+
+        g.set_enum('status_code', self.enum.PENDING_PRODUCT_STATUS)
+
+        g.set_sort_defaults('created', 'desc')
+
+        g.set_link('description')
+
+    def configure_form(self, f):
+        super(PendingProductView, self).configure_form(f)
+        model = self.model
+        pending = f.model_instance
+
+        # product key
+        if '_product_key_' in f:
+            key = self.rattail_config.product_key()
+            f.replace('_product_key_', key)
+            f.set_label(key, self.rattail_config.product_key_title(key))
+            f.set_renderer(key, self.render_product_key_value)
+
+        # department
+        if self.creating or self.editing:
+            if 'department' in f:
+                f.remove('department_name')
+                f.replace('department', 'department_uuid')
+                f.set_widget('department_uuid', forms.widgets.DepartmentWidget(self.request, required=False))
+                f.set_label('department_uuid', "Department")
+        else:
+            f.set_renderer('department', self.render_department)
+            if pending.department:
+                f.remove('department_name')
+
+        # brand
+        if self.creating or self.editing:
+            f.remove('brand_name')
+            f.replace('brand', 'brand_uuid')
+            f.set_label('brand_uuid', "Brand")
+
+            f.set_node('brand_uuid', colander.String(), missing=colander.null)
+            brand_display = ""
+            if self.request.method == 'POST':
+                if self.request.POST.get('brand_uuid'):
+                    brand = self.Session.query(model.Brand).get(self.request.POST['brand_uuid'])
+                    if brand:
+                        brand_display = six.text_type(brand)
+            elif self.editing:
+                brand_display = six.text_type(pending.brand or '')
+            brands_url = self.request.route_url('brands.autocomplete')
+            f.set_widget('brand_uuid', forms.widgets.JQueryAutocompleteWidget(
+                field_display=brand_display, service_url=brands_url))
+        else:
+            f.set_renderer('brand', self.render_brand)
+            if pending.brand:
+                f.remove('brand_name')
+
+        # description
+        f.set_required('description')
+
+        # case_size
+        f.set_type('case_size', 'quantity')
+
+        # regular_price_amount
+        f.set_type('regular_price_amount', 'currency')
+
+        # notes
+        f.set_type('notes', 'text')
+
+        # created
+        if self.creating:
+            f.remove('created')
+        else:
+            f.set_readonly('created')
+
+        # user
+        if self.creating:
+            f.remove('user')
+        else:
+            f.set_readonly('user')
+            f.set_renderer('user', self.render_user)
+
+        # status_code
+        if self.creating:
+            f.remove('status_code')
+        else:
+            # f.set_readonly('status_code')
+            f.set_enum('status_code', self.enum.PENDING_PRODUCT_STATUS)
+
+    def objectify(self, form, data=None):
+        if data is None:
+            data = form.validated
+
+        pending = super(PendingProductView, self).objectify(form, data)
+
+        if not pending.user:
+            pending.user = self.request.user
+
+        self.Session.add(pending)
+        self.Session.flush()
+        self.Session.refresh(pending)
+
+        if pending.department:
+            pending.department_name = pending.department.name
+
+        if pending.brand:
+            pending.brand_name = pending.brand.name
+
+        return pending
 
 
 def print_labels(request):
@@ -1864,12 +2148,9 @@ def print_labels(request):
 
 def includeme(config):
 
-    config.add_route('products.autocomplete', '/products/autocomplete')
-    config.add_view(ProductsAutocomplete, route_name='products.autocomplete',
-                    renderer='json', permission='products.list')
-
     config.add_route('products.print_labels', '/products/labels')
     config.add_view(print_labels, route_name='products.print_labels',
                     renderer='json', permission='products.print_labels')
 
     ProductView.defaults(config)
+    PendingProductView.defaults(config)
