@@ -218,8 +218,10 @@ class PurchasingBatchView(BatchMasterView):
 
     def configure_form(self, f):
         super(PurchasingBatchView, self).configure_form(f)
+        model = self.model
         batch = f.model_instance
         today = localtime(self.rattail_config).date()
+        use_buefy = self.get_use_buefy()
 
         # mode
         f.set_enum('mode', self.enum.PURCHASE_BATCH_MODE)
@@ -313,6 +315,36 @@ class PurchasingBatchView(BatchMasterView):
                     field_display=buyer_display, service_url=buyers_url))
                 f.set_label('buyer_uuid', "Buyer")
 
+        # invoice_file
+        if self.creating:
+            f.set_type('invoice_file', 'file', required=False)
+        else:
+            f.set_readonly('invoice_file')
+            f.set_renderer('invoice_file', self.render_downloadable_file)
+
+        # invoice_parser_key
+        if self.creating:
+            kwargs = {}
+
+            if 'vendor_uuid' in self.request.matchdict:
+                vendor = self.Session.query(model.Vendor).get(
+                    self.request.matchdict['vendor_uuid'])
+                if vendor:
+                    kwargs['vendor'] = vendor
+
+            parsers = self.handler.get_supported_invoice_parsers(**kwargs)
+            parser_values = [(p.key, p.display) for p in parsers]
+            if len(parsers) == 1:
+                f.set_default('invoice_parser_key', parsers[0].key)
+
+            if use_buefy:
+                f.set_widget('invoice_parser_key', dfwidget.SelectWidget(values=parser_values))
+            else:
+                parser_values.insert(0, ('', "(please choose)"))
+                f.set_widget('invoice_parser_key', forms.widgets.JQuerySelectWidget(values=parser_values))
+        else:
+            f.remove_field('invoice_parser_key')
+
         # date_ordered
         f.set_type('date_ordered', 'date_jquery')
         if self.creating:
@@ -582,8 +614,11 @@ class PurchasingBatchView(BatchMasterView):
         g.set_type('po_total_calculated', 'currency')
         g.set_type('credits', 'boolean')
 
-        # we only want the grid column to have abbreviated label, but *not* the filter
+        # we only want the grid columns to have abbreviated labels,
+        # but *not* the filters
         # TODO: would be nice to somehow make this simpler
+        g.set_label('department_name', "Department")
+        g.filters['department_name'].label = "Department Name"
         g.set_label('cases_ordered', "Cases Ord.")
         g.filters['cases_ordered'].label = "Cases Ordered"
         g.set_label('units_ordered', "Units Ord.")
@@ -597,6 +632,16 @@ class PurchasingBatchView(BatchMasterView):
         g.set_label('units_received', "Units Rec.")
         g.filters['units_received'].label = "Units Received"
 
+        # catalog_unit_cost
+        g.set_renderer('catalog_unit_cost', self.render_row_grid_cost)
+        g.set_label('catalog_unit_cost', "Catalog Cost")
+        g.filters['catalog_unit_cost'].label = "Catalog Unit Cost"
+
+        # invoice_unit_cost
+        g.set_renderer('invoice_unit_cost', self.render_row_grid_cost)
+        g.set_label('invoice_unit_cost', "Invoice Cost")
+        g.filters['invoice_unit_cost'].label = "Invoice Unit Cost"
+
         # invoice_total
         g.set_type('invoice_total', 'currency')
         g.set_label('invoice_total', "Total")
@@ -608,6 +653,16 @@ class PurchasingBatchView(BatchMasterView):
         g.set_label('po_total', "Total")
         g.set_label('credits', "Credits?")
 
+        g.set_link('upc')
+        g.set_link('vendor_code')
+        g.set_link('description')
+
+    def render_row_grid_cost(self, row, field):
+        cost = getattr(row, field)
+        if cost is None:
+            return ""
+        return "{:0,.3f}".format(cost)
+
     def make_row_grid_tools(self, batch):
         return self.make_default_row_grid_tools(batch)
 
@@ -620,7 +675,11 @@ class PurchasingBatchView(BatchMasterView):
                                row.STATUS_ORDERED_RECEIVED_DIFFER,
                                row.STATUS_TRUCKDUMP_UNCLAIMED,
                                row.STATUS_TRUCKDUMP_PARTCLAIMED,
-                               row.STATUS_OUT_OF_STOCK):
+                               row.STATUS_OUT_OF_STOCK,
+                               row.STATUS_ON_PO_NOT_INVOICE,
+                               row.STATUS_ON_INVOICE_NOT_PO,
+                               row.STATUS_COST_INCREASE,
+                               row.STATUS_DID_NOT_RECEIVE):
             return 'notice'
 
     def configure_row_form(self, f):
@@ -650,10 +709,10 @@ class PurchasingBatchView(BatchMasterView):
         f.set_type('units_mispick', 'quantity')
 
         # currency fields
-        f.set_type('po_unit_cost', 'currency')
+        # nb. we only show "total" fields as currency, but not case or
+        # unit cost fields, b/c currency is rounded to 2 places
         f.set_type('po_total', 'currency')
         f.set_type('po_total_calculated', 'currency')
-        f.set_type('invoice_unit_cost', 'currency')
 
         # upc
         f.set_type('upc', 'gpc')
