@@ -38,7 +38,6 @@ from rattail import pod
 from rattail.db import model, Session as RattailSession
 from rattail.time import localtime, make_utc
 from rattail.util import pretty_quantity, prettify, OrderedDict, simple_error
-from rattail.vendors.invoices import iter_invoice_parsers, require_invoice_parser
 from rattail.threads import Thread
 
 import colander
@@ -81,7 +80,6 @@ class ReceivingBatchView(PurchasingBatchView):
         'truck_dump',
         'description',
         'department',
-        'buyer',
         'date_ordered',
         'created',
         'created_by',
@@ -203,6 +201,12 @@ class ReceivingBatchView(PurchasingBatchView):
     def batch_mode(self):
         return self.enum.PURCHASE_BATCH_MODE_RECEIVING
 
+    def configure_grid(self, g):
+        super(ReceivingBatchView, self).configure_grid(g)
+
+        if not self.handler.allow_truck_dump_receiving():
+            g.remove('truck_dump')
+
     def create(self, form=None, **kwargs):
         """
         Custom view for creating a new receiving batch.  We split the process
@@ -210,6 +214,10 @@ class ReceivingBatchView(PurchasingBatchView):
         form details for creating a batch will depend on which "type" of batch
         creation is to be done, and it's much easier to keep conditional logic
         for that in the server instead of client-side etc.
+
+        See also
+        :meth:`tailbone.views.purchasing.costing:CostingBatchView.create()`
+        which uses similar logic.
         """
         route_prefix = self.get_route_prefix()
         workflows = self.handler.supported_receiving_workflows()
@@ -440,25 +448,6 @@ class ReceivingBatchView(PurchasingBatchView):
                             'truck_dump_status',
                             'truck_dump_batch')
 
-        # invoice_file
-        if self.creating:
-            f.set_type('invoice_file', 'file', required=False)
-        else:
-            f.set_readonly('invoice_file')
-            f.set_renderer('invoice_file', self.render_downloadable_file)
-
-        # invoice_parser_key
-        if self.creating:
-            parsers = sorted(iter_invoice_parsers(), key=lambda p: p.display)
-            parser_values = [(p.key, p.display) for p in parsers]
-            parser_values.insert(0, ('', "(please choose)"))
-            if use_buefy:
-                f.set_widget('invoice_parser_key', dfwidget.SelectWidget(values=parser_values))
-            else:
-                f.set_widget('invoice_parser_key', forms.widgets.JQuerySelectWidget(values=parser_values))
-        else:
-            f.remove_field('invoice_parser_key')
-
         # store
         if self.creating:
             store = self.rattail_config.get_store(self.Session())
@@ -468,7 +457,7 @@ class ReceivingBatchView(PurchasingBatchView):
             f.set_widget('store_uuid', dfwidget.HiddenWidget())
 
         # purchase
-        if (self.creating and workflow == 'from_po'
+        if (self.creating and workflow in ('from_po', 'from_po_with_invoice')
             and self.purchase_order_fieldname == 'purchase'):
             if use_buefy:
                 f.replace('purchase', 'purchase_uuid')
@@ -518,6 +507,11 @@ class ReceivingBatchView(PurchasingBatchView):
                          'invoice_file',
                          'invoice_parser_key')
 
+            elif workflow == 'from_po_with_invoice':
+                f.remove('truck_dump_batch_uuid')
+                f.set_required('invoice_file')
+                f.set_required('invoice_parser_key')
+
             elif workflow == 'truck_dump_children_first':
                 f.remove('truck_dump_batch_uuid',
                          'invoice_file',
@@ -572,6 +566,9 @@ class ReceivingBatchView(PurchasingBatchView):
         elif batch_type == 'from_po':
             # TODO: how to best handle this field?  this doesn't seem flexible
             kwargs['purchase_key'] = batch.purchase_uuid
+        elif batch_type == 'from_po_with_invoice':
+            # TODO: how to best handle this field?  this doesn't seem flexible
+            kwargs['purchase_key'] = batch.purchase_uuid
         elif batch_type == 'truck_dump_children_first':
             kwargs['truck_dump'] = True
             kwargs['truck_dump_children_first'] = True
@@ -737,22 +734,11 @@ class ReceivingBatchView(PurchasingBatchView):
 
     def configure_row_grid(self, g):
         super(ReceivingBatchView, self).configure_row_grid(g)
-        g.set_label('department_name', "Department")
 
         # vendor_code
         g.filters['vendor_code'].default_active = True
         g.filters['vendor_code'].default_verb = 'contains'
 
-        # catalog_unit_cost
-        g.set_renderer('catalog_unit_cost', self.render_row_grid_cost)
-        g.set_label('catalog_unit_cost', "Catalog Cost")
-        g.filters['catalog_unit_cost'].label = "Catalog Unit Cost"
-
-        # invoice_unit_cost
-        g.set_renderer('invoice_unit_cost', self.render_row_grid_cost)
-        g.set_label('invoice_unit_cost', "Invoice Cost")
-        g.filters['invoice_unit_cost'].label = "Invoice Unit Cost"
-
         # credits
         # note that sorting by credits involves a subquery with group by clause.
         # seems likely there may be a better way? but this seems to work fine
@@ -800,12 +786,6 @@ class ReceivingBatchView(PurchasingBatchView):
 
         return css_class
 
-    def render_row_grid_cost(self, row, field):
-        cost = getattr(row, field)
-        if cost is None:
-            return ""
-        return "{:0,.3f}".format(cost)
-
     def transform_unit_url(self, row, i):
         # grid action is shown only when we return a URL here
         if self.row_editable(row):
