@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2019 Lance Edgar
+#  Copyright © 2010-2021 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -36,7 +36,6 @@ import rattail
 from rattail.db import model, Session as RattailSession
 from rattail.files import resource_path
 from rattail.time import localtime
-from rattail.reporting import get_report_handler
 from rattail.threads import Thread
 from rattail.util import simple_error, OrderedDict
 
@@ -46,7 +45,7 @@ from mako.template import Template
 from pyramid.response import Response
 from webhelpers2.html import HTML
 
-from tailbone import forms, grids
+from tailbone import forms
 from tailbone.db import Session
 from tailbone.views import View
 from tailbone.views.exports import ExportMasterView
@@ -258,18 +257,39 @@ class ReportOutputView(ExportMasterView):
         params.sort(key=lambda param: param['key'])
 
         route_prefix = self.get_route_prefix()
-        g = grids.Grid(
+        factory = self.get_grid_factory()
+        g = factory(
             key='{}.params'.format(route_prefix),
             data=params,
             columns=['key', 'value'],
+            labels={'key': "Name"},
         )
-        return HTML.literal(g.render_grid())
+        if self.get_use_buefy():
+            return HTML.literal(
+                g.render_buefy_table_element(data_prop='paramsData'))
+        else:
+            return HTML.literal(g.render_grid())
 
     def render_download(self, report, field):
         path = report.filepath(self.rattail_config)
         url = self.get_action_url('download', report)
         return self.render_file_field(path, url=url)
 
+    def template_kwargs_view(self, **kwargs):
+        use_buefy = self.get_use_buefy()
+        if use_buefy:
+
+            report = kwargs['instance']
+            params_data = []
+            for name, value in report.params.items():
+                params_data.append({
+                    'key': name,
+                    'value': value,
+                })
+            kwargs['params_data'] = params_data
+
+        return kwargs
+
     def download(self):
         report = self.get_instance()
         path = report.filepath(self.rattail_config)
@@ -286,7 +306,8 @@ class GenerateReport(View):
         self.handler = self.get_handler()
 
     def get_handler(self):
-        return get_report_handler(self.rattail_config)
+        app = self.get_rattail_app()
+        return app.get_report_handler()
 
     def choose(self):
         """
@@ -313,7 +334,7 @@ class GenerateReport(View):
         values = [(r.type_key, r.name) for r in reports.values()]
         values.sort(key=lambda r: r[1])
         if use_buefy:
-            form.set_widget('report_type', forms.widgets.CustomSelectWidget(values=values, size=10))
+            form.set_widget('report_type', forms.widgets.CustomSelectWidget(values=values))
             form.widgets['report_type'].set_template_values(input_handler='reportTypeChanged')
         else:
             form.set_widget('report_type', forms.widgets.PlainSelectWidget(values=values, size=10))
@@ -387,6 +408,13 @@ class GenerateReport(View):
             if param.type is datetime.date:
                 form.set_type(param.name, 'date_jquery')
 
+        # auto-select default choice for fields which have only one
+        for param in report_params:
+            if param.type == 'choice' and param.required:
+                values = form.schema[param.name].widget.values
+                if len(values) == 1:
+                    form.set_default(param.name, values[0][0])
+
         # if form validates, start generating new report output; show progress page
         if form.validate(newstyle=True):
             key = 'report_output.generate'
