@@ -46,6 +46,12 @@ class SubdepartmentView(MasterView):
         'department',
     ]
 
+    form_fields = [
+        'number',
+        'name',
+        'department',
+    ]
+
     mergeable = True
     merge_additive_fields = [
         'product_count',
@@ -57,6 +63,23 @@ class SubdepartmentView(MasterView):
         'department_number',
     ]
 
+    has_rows = True
+    model_row_class = model.Product
+
+    row_labels = {
+        'upc': "UPC",
+    }
+
+    row_grid_columns = [
+        'upc',
+        'brand',
+        'description',
+        'size',
+        'vendor',
+        'regular_price',
+        'current_price',
+    ]
+
     def configure_grid(self, g):
         super(SubdepartmentView, self).configure_grid(g)
 
@@ -98,6 +121,33 @@ class SubdepartmentView(MasterView):
 
         Session.delete(removing)
 
+    def get_row_data(self, subdepartment):
+        return self.Session.query(model.Product)\
+                           .filter(model.Product.subdepartment == subdepartment)
+
+    def get_parent(self, product):
+        return product.subdepartment
+
+    def configure_row_grid(self, g):
+        super(SubdepartmentView, self).configure_row_grid(g)
+
+        app = self.get_rattail_app()
+        self.handler = app.get_products_handler()
+        g.set_renderer('regular_price', self.render_price)
+        g.set_renderer('current_price', self.render_price)
+
+        g.set_sort_defaults('upc')
+
+    def render_price(self, product, field):
+        if not product.not_for_sale:
+            price = product[field]
+            if price:
+                return self.handler.render_price(price)
+
+    def row_view_action_url(self, product, i):
+        return self.request.route_url('products.view', uuid=product.uuid)
+
+
 # TODO: deprecate / remove this
 SubdepartmentsView = SubdepartmentView
 
