@@ -32,7 +32,8 @@ import six
 from sqlalchemy import orm
 
 from rattail.db import model
-from rattail.db.auth import administrator_role, guest_role, authenticated_role, set_user_password, has_permission
+from rattail.db.auth import (administrator_role, guest_role,
+                             authenticated_role, set_user_password)
 
 import colander
 from deform import widget as dfwidget
@@ -52,6 +53,7 @@ class UserView(PrincipalMasterView):
     has_rows = True
     model_row_class = model.UserEvent
     has_versions = True
+    touchable = True
 
     grid_columns = [
         'username',
@@ -162,6 +164,17 @@ class UserView(PrincipalMasterView):
         if query.count():
             raise colander.Invalid(node, "Username must be unique")
 
+    def valid_person(self, node, value):
+        """
+        Make sure ``value`` corresponds to an existing
+        ``Person.uuid``.
+        """
+        if value:
+            model = self.model
+            person = self.Session.query(model.Person).get(value)
+            if not person:
+                raise colander.Invalid(node, "Person not found (you must *select* a record)")
+
     def configure_form(self, f):
         super(UserView, self).configure_form(f)
         user = f.model_instance
@@ -186,6 +199,7 @@ class UserView(PrincipalMasterView):
                 people_url = self.request.route_url('people.autocomplete')
                 f.set_widget('person_uuid', forms.widgets.JQueryAutocompleteWidget(
                     field_display=person_display, service_url=people_url))
+                f.set_validator('person_uuid', self.valid_person)
                 f.set_label('person_uuid', "Person")
 
         # person name(s)
@@ -217,6 +231,8 @@ class UserView(PrincipalMasterView):
                 size = len(roles)
                 if size < 3:
                     size = 3
+                elif size > 20:
+                    size = 20
                 f.set_widget('roles', dfwidget.SelectWidget(multiple=True,
                                                             size=size,
                                                             values=role_values))
@@ -236,7 +252,8 @@ class UserView(PrincipalMasterView):
         if self.viewing:
             permissions = self.request.registry.settings.get('tailbone_permissions', {})
             f.append('permissions')
-            f.set_renderer('permissions', PermissionsRenderer(permissions=permissions,
+            f.set_renderer('permissions', PermissionsRenderer(request=self.request,
+                                                              permissions=permissions,
                                                               include_guest=True,
                                                               include_authenticated=True))
 
@@ -318,6 +335,14 @@ class UserView(PrincipalMasterView):
                 if self.request.is_root or uuid != admin.uuid:
                     user._roles.append(model.UserRole(role_uuid=uuid))
 
+                    # also record a change to the role, for datasync.
+                    # this is done "just in case" the role is to be
+                    # synced to all nodes
+                    if self.Session().rattail_record_changes:
+                        self.Session.add(model.Change(class_name='Role',
+                                                      instance_uuid=uuid,
+                                                      deleted=False))
+
         # remove any roles which were *not* specified, although must take care
         # not to remove admin role, unless acting as root
         for uuid in old_roles:
@@ -326,6 +351,14 @@ class UserView(PrincipalMasterView):
                     role = self.Session.query(model.Role).get(uuid)
                     user.roles.remove(role)
 
+                    # also record a change to the role, for datasync.
+                    # this is done "just in case" the role is to be
+                    # synced to all nodes
+                    if self.Session().rattail_record_changes:
+                        self.Session.add(model.Change(class_name='Role',
+                                                      instance_uuid=uuid,
+                                                      deleted=False))
+
     def render_person(self, user, field):
         person = user.person
         if not person:
@@ -370,6 +403,9 @@ class UserView(PrincipalMasterView):
         ]
 
     def find_principals_with_permission(self, session, permission):
+        app = self.get_rattail_app()
+        auth = app.get_auth_handler()
+
         # TODO: this should search Permission table instead, and work backward to User?
         all_users = session.query(model.User)\
                            .filter(model.User.active == True)\
@@ -379,7 +415,7 @@ class UserView(PrincipalMasterView):
                                     .joinedload(model.Role._permissions))
         users = []
         for user in all_users:
-            if has_permission(session, user, permission):
+            if auth.has_permission(session, user, permission):
                 users.append(user)
         return users
 
