@@ -26,9 +26,7 @@ Views pertaining to vendors
 
 from __future__ import unicode_literals, absolute_import
 
-from .core import VendorView, VendorsAutocomplete
-# TODO: deprecate / remove this
-from .core import VendorsView
+from .core import VendorView
 
 
 def includeme(config):
